/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2020 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * blocked.p
 *
 *
 * Blocked clients.
 *
 *
 * To Do:
 *
 *	Handle other sorts of blocked clients
 *	Ought to break up "blkstr" into it's components
 *
 *
 * Author:
 *
 *	Tom Bascom, Greenfield Technologies
 *	http://www.greenfieldtech.com
 *	September 16, 2003
 *
 */

{lib/protop.i}

define output parameter dcDescription as character no-undo initial "Blocked".

define temp-table tt_blocked no-undo
  field xid       as integer
  field xvalid    as logical

  field usrnum    as integer   format ">>>>9"     label "Usr"
  field userName  as character format "x(15)"     label "Name"
  field userPID   as character format "x(8)"      label "PID"
  field userFlags as character format "x(5)"      label "Flags"
  field blkDura   as character format "x(8)"      label "Duration"
  field blkWait   as character format "x(8)"      label "Wait"
  field blkResrc  as character format "x(12)"     label "Resrc Id"
  field blkNote   as character format "x(87)"     label "Table Blocker-Usr#:Device:PID Blocker-StatementCache WaitList"		/* beware! hard-coded width 87 is used below	*/
  field blkBy     as integer   format ">>>>9"     label "BlkBy"

  field zblkTime  as datetime                     label "Blkd Since"
  field zBlkDura  as integer   format ">>>>>>>>9" label "Duration"
  field zBlkrQ    as integer   format ">>>>9"     label "BlockerQ"	/* how deep is the Q being blocked by this bozo?	*/

  index xid-idx     is unique xid
  index blkDura-idx is primary blkDura descending
  index xid-blkBy   blkBy
  index xid-zBlkrQ  zBlkrQ
.

define temp-table tt_blocker no-undo
  field xid      as integer
  field xvalid   as logical
  field stackNum as integer   format ">>>>>>>9"  label "Depth"
  field lineNum  as integer   format ">>>>>>>9"  label "Line#"
  field procName as character format "x(137)"    label "Procedure"
  index stackNum-idx is primary /* usrNum */ stackNum ascending
  index xid-idx     is unique xid
.

define temp-table tt_blocker_Info no-undo
  field infoString as character
  index infoString-idx is primary unique infoString
.

define new global shared temp-table tt_lock no-undo like _lock			/* lib/vstlib.p					*/
  field qflag as logical
  index id-idx is unique primary _lock-id
  index recid-idx _lock-recid
  index usr-idx _lock-usr
  index tbl-idx _lock-table
.

define new global shared variable dbgMode  as integer no-undo initial 1.	/* 1 = errors, 5 = info, 9 = response to file	*/
define new global shared variable rowLimit as integer no-undo.


procedure mon-init:

  define variable b as handle.

  b = ( temp-table tt_blocked:default-buffer-handle ).

  if pt_altWidth = "80" then
    do:
      altFormat( "80", b, "usrNum",   ">>>9" ).
      altFormat( "80", b, "userName",  "x(15)" ).
      altFormat( "80", b, "blkDura",   "x(8)" ).
      altFormat( "80", b, "blkWait",   "x(8)" ).
      altFormat( "80", b, "blkResrc",  "x(12)" ).
      altFormat( "80", b, "blkNote",   "x(28)" ).
    end.

  empty temp-table tt_blocked.

  return.

end.

/* update
 *
 */

procedure mon-update:

  define input parameter argList as character no-undo.

  define variable u_name  as character no-undo.
  define variable u_flags as character no-undo.
  define variable u_pid   as character no-undo.
  define variable u_csc   as character no-undo.
  define variable bstr    as character no-undo.

  define variable xusr    as character no-undo.
  define variable uInfo   as character no-undo.

  define variable lkScan  as integer   no-undo.
  define variable i       as integer   no-undo.
  define variable j       as integer   no-undo.
  define variable t       as integer   no-undo.

  define variable ci as character no-undo.
  define variable cl as integer   no-undo.

  for each tt_blocked:
    tt_blocked.xvalid = no.
  end.

  empty temp-table tt_blocker.

  empty temp-table tt_Lock.
  lkscan = 0.

  /* get a list of tt_blocked sessions
   *
   */

  for each dictdb._Connect no-lock
    where _Connect-usr <> ?
      and _Connect-wait <> " -- ":					/* not perfect -- but works for record locks 		*/

    i = i + 1.

    if i = 1 then snapshotLock().			/* there is at least one blocked user... grab a snapshot of _Lock	*/

    find tt_blocked where tt_blocked.usrnum = _Connect-usr no-error.

    if not available tt_blocked then
      do:
        create tt_blocked.
        assign
          tt_blocked.userPID   = string( _connect-PID )			/* UDF has to be before indexed fields in old releases	*/
          tt_blocked.userFlags = connectFlags( _Connect-Id )
          tt_blocked.userName  = connectName( _Connect-Id, tt_blocked.userFlags )
          tt_blocked.xid       = _Connect-usr 
          tt_blocked.usrnum    = _Connect-usr 
          tt_blocked.zblkTime  = now
       .
      end.

    if tt_blocked.blkResrc <> string( _Connect-wait1 ) then		/* if wait1 has changed it is a new event!  		*/
      tt_blocked.zblkTime = now.

    /* "96" = sequence block in the schema area of a 4k database */

    assign
      tt_blocked.xvalid    = yes
      tt_blocked.zBlkDura  = abs( interval( now, tt_blocked.zBlkTime, "seconds" ))
      tt_blocked.blkDura   = string( tt_blocked.zBlkDura, "hh:mm:ss" )
      tt_blocked.blkWait   = _Connect-Wait
      tt_blocked.blkResrc  = string( _Connect-wait1 )
      tt_blocked.blkBy     = ?
      tt_blocked.zBlkrQ    = 0
    .

    /* use the snapshot of _Lock that was made above -- _Lock is not 100% reliable and records may be missing etc.
     *
     * RECIDs are not unique -- so the FIND /could/ find more than one lock meeting the criteria and thus be unable to
     * determine what we are actually waiting for :( and who else might be holding it.
     *
     * "FIRST" would "fix" that in that the query would succeed but that would be misleading (at best) or just plain wrong.
     *
     * IF we find a unique record (available = true) then we can be reasonably confident that it is the one that the user
     * is waiting for so we show the table name and proceed to find out more about the holder of the record lock with getUserInfo() 
     *
     */

    /* wait2 is the table# as of at least 12.0
     */

    find /* first */ tt_lock where tt_lock._lock-usr = _connect-usr and tt_lock._lock-recid = _connect-wait1 no-error.
    if not available tt_lock then
      do:
        if dbgMode >= 5 then message now substitute( "unable to find a lock for usr: &1 recid: &2", _connect-usr, _connect-wait1 ).
        /* pause. */
      end.
     else
      do:

        t = tt_lock._lock-table.

        find tt_tbl no-lock where tt_tbl.xid = t no-error.

        /* https://knowledgebase.progress.com/articles/Knowledge/21639 - info re: _lock-flags	*/
        /* https://knowledgebase.progress.com/articles/Article/P6854 - info: re table locks	*/

        assign
          tt_blocked.blkWait = _Connect-Wait + " " + ( if tt_lock._lock-flags = ? then "" else replace( tt_lock._lock-flags, " ", "" ))
          tt_blocked.blkNote = ( if available tt_tbl then tt_tbl.tblName else substitute( "[&1]", t ))
        .

        for each tt_lock
           where tt_lock._lock-table = t
             and tt_lock.qflag       = no			/* not a queued lock */
             and tt_lock._lock-recid = _connect-wait1
             and tt_lock._lock-usr  <> _connect-usr:

          uInfo = getUserInfo( tt_lock._lock-usr ).
          if not( uInfo begins  ":" ) then tt_blocked.blkBy = integer( entry( 1, uInfo, ":" )).
          tt_blocked.blkNote = tt_blocked.blkNote + " " + uInfo.
          if length( tt_blocked.blkNote ) > 87 then tt_blocked.blkNote = substring( tt_blocked.blkNote, 1, 85 ) + "...".	/* 87 and 85 are related to format above!	*/

        end.    

        /* add a list of queued users...
         */

        j = 0.

        for each tt_lock
           where tt_lock._lock-table = t
             and tt_lock.qflag       = yes
             and tt_lock._lock-recid = _connect-wait1
             and tt_lock._lock-usr  <> _connect-usr:

          xusr = string( tt_lock._lock-usr ).

          tt_blocked.blkNote = tt_blocked.blkNote + " " + xusr.

          j = j + 1.
          if j > 5 then leave.

        end.    

      end.	/* available tt_lock	*/

    if ( rowLimit > 0 ) and ( i >= rowLimit ) then leave.

  end.

  for each tt_blocked where tt_blocked.xvalid = no:
    delete tt_blocked.
  end.

  /* how may processes are being blocked by each blocker?
   */

  for each tt_blocked break by blkBy:
    if first-of( blkBy ) then i = 0.
    i = i + 1.
    if last-of( blkBy ) then zBlkrQ = i.
  end.

  for each tt_blocked:
  end.

  add2ds( temp-table tt_blocked:default-buffer-handle ).

/***
 *   add2ds( temp-table tt_blocker:default-buffer-handle ).
 *   add2ds( temp-table tt_blocker_Info:default-buffer-handle ).
 ***/

  return.

end.

{ssg/blocked.i}
{ssg/blocker.i}

return.
