/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2020 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * dbid.p
 *
 */

{lib/protop.i}
{lib/protoplib.i}

define output parameter dcDescription as character no-undo initial "DBId".

define temp-table tt_DBId no-undo

  field dbidLogNm       as character label "DB Logical Name"
  field dbidPhysNm      as character label "DB Physical Name"
  field dbidHost        as character label "DB Host Name"
  field dbidEnvironment as character label "DB Environment"

  field pt_dbAccess   as int64
  field pt_dbRead     as int64

  field pt_propath    as character label "PT Propath"
  field pt_dlc        as character label "PT DLC"
  field pt_protop     as character label "$PROTOP"

  index dbidLogNm-idx is unique primary dbidLogNm
.

define new global shared variable pt_envDescription as character no-undo.


procedure mon-init:
  return.
end.


define variable dbidx as character no-undo.

procedure mon-update:

  define input parameter argList as character no-undo.

  find tt_dbid no-error.
  if not available( tt_dbid ) then create tt_dbid.

  if tt_dbid.dbidHost = "" or tt_dbid.dbidHost = ? then
    tt_dbid.dbidHost = getHostName().

  tt_dbid.pt_propath = propath.

  if tt_dbid.pt_dlc = "" then
    tt_dbid.pt_dlc = os-getenv( "DLC" ).

  if tt_dbid.pt_protop = "" then
    tt_dbid.pt_protop = os-getenv( "PROTOP" ).

  if connected( "dictdb" ) then
    assign
      tt_dbid.dbidLogNm    = ldbName( "DICTDB" )
      tt_dbid.dbidPhysNm   = pdbName( "DICTDB" )
      tt_dbid.dbidEnvironment = tt_dbid.dbidPhysNm
    .
   else
    assign
      tt_dbid.dbidLogNm    = "n/a"
      tt_dbid.dbidPhysNm   = "n/a"
      tt_dbid.dbidEnvironment = pt_shortName
    .

  if pt_envDescription <> ? and pt_envDescription <> "" then
    tt_dbid.dbidEnvironment = substitute( "&1 &2 &1", pt_envDescription, tt_dbid.dbidEnvironment ).

  dbidx = tt_dbid.dbidLogNm.

  add2ds( temp-table tt_dbid:default-buffer-handle ).

  return.

end.

define new global shared variable programmerMode as logical  no-undo.

procedure updPTStats:

  define input parameter myId  as integer   no-undo.
  define input parameter logRd as int64 no-undo.
  define input parameter osRd  as int64 no-undo.

  find tt_dbid no-error.
  if not available( tt_dbid ) then create tt_dbid.
  assign
    pt_dbAccess = logRd		/* this only tracks *real* tables, access to pseudo tables such as VSTs have no impact on _dbAccess and are not tracked anywhere	*/
    pt_dbRead   = osRd
/* +++    tt_dbid.dbidLogNm  = substitute( "&1 &2 &3 &4 &5", dbidx, myId - 1, logRd, osRd, ( if programmerMode = yes then "programmerMode" else "autoSample" ))	*/
    tt_dbid.dbidLogNm  = ( if programmerMode = yes then "programmerMode" else "autoSample" )
    tt_dbid.dbidPhysNm = pdbName( "DICTDB" )
  .


/***
      replace(
        trim( substitute( "&1 &2 &3 &4 &5 &6 &7 &8 &9",
          sampleMode,
          entry( rawMode, "Initial,Previous,Raw,Cumulative,Interval" ),
          entry( timeMode, "Summary,Rate" ),
          rt,
          dsRows,	/*** @@@ ***/
          dsSize,
          trim( string( fetchTime / 1000, ">>>9.999" )),
          ( if logMode then "Log=On" else "" )
        )),
        "  ",
        " "
      )

 ***/

  tt_dbid.pt_propath = propath.

  if tt_dbid.pt_dlc = "" then
    tt_dbid.pt_dlc = os-getenv( "DLC" ).

  if tt_dbid.pt_protop = "" then
    tt_dbid.pt_protop = os-getenv( "PROTOP" ).

  if connected( "dictdb" ) then
    assign
      tt_dbid.dbidLogNm    = ldbName( "DICTDB" )
      tt_dbid.dbidPhysNm   = pdbName( "DICTDB" )
      tt_dbid.dbidEnvironment = tt_dbid.dbidPhysNm
    .
   else
    assign
      tt_dbid.dbidLogNm    = "n/a"
      tt_dbid.dbidPhysNm   = "n/a"
      tt_dbid.dbidEnvironment = pt_shortName
    .

  if pt_envDescription <> ? and pt_envDescription <> "" then
    tt_dbid.dbidEnvironment = substitute( "&1 &2 &1", pt_envDescription, tt_dbid.dbidEnvironment ).

  return.

end.

subscribe to "updPTStats" anywhere run-procedure "updPTStats".

return.
