/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2020 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * tblact.p
 *
 *
 * Table Activity
 *
 *
 * Author:
 *
 *	Tom Bascom, Greenfield Technologies
 *	http://www.greenfieldtech.com
 *	September 5, 2003
 *
 */

{lib/protop.i}
{lib/tick.i}

define output parameter dcDescription as character no-undo initial "TableActivity".

define variable hasOSRd as logical no-undo.

{lib/tt_table.i}

define temp-table tt_tblAct no-undo
  field xid        as integer   format "->>>>9"
  field tblNum     as integer   format "->>>>9"          label "Tbl#"
  field tblAreaNum as integer   format ">>>>9"           label "Area#"
  field tblPool    as character format "x(3)"            label "BX"
  field tblName    as character format "x(30)"           label "Table Name"
  field dbaRM      as decimal   format ">>>>>>>>>>>>"    label "RM Chain"
  field dbaRecs    as decimal   format ">>>>>>>>>>>>"    label "#Records"
  field dbaFragPct as decimal   format ">>>9.99%"        label "Frag%"
  field dbaScatter as decimal   format ">>>>>>"          label "Scat"
  field tblTurn    as decimal   format ">>>>>>9.99"      label "Churn"
  field dbaAvgRow  as decimal   format ">>>>>>>"         label "AvgRow"
  field tblCr      as decimal   format ">>>>>>>>9"       label "Create"
  field tblRd      as decimal   format ">>>>>>>>>9"      label "Read"
  field tblUp      as decimal   format ">>>>>>>>9"       label "Update"
  field tblDl      as decimal   format ">>>>>>>>9"       label "Delete"
  field tblOSRd    as decimal   format ">>>>>>>>9"       label "OS Read"

  index xid-idx    is unique xid
  index tblRd-idx  is primary tblRd descending
.


define variable baseTbl  as integer no-undo.
define variable tblRange as integer no-undo.

function chkTblRange returns logical ():

  define variable tBase  as integer no-undo.
  define variable tRange as integer no-undo.

  find first dictdb._tablestat no-lock.
  tBase = dictdb._tablestat._tablestat-id.
 
  find last dictdb._tablestat no-lock.
  tRange = dictdb._tablestat._tablestat-id - tBase + 1.

  if tBase = baseTbl and tRange = tblRange then
    return true.
   else
    do:
      assign
        baseTbl  = tBase
        tblRange = tRange
      .
      return false.
    end.

end.


procedure chkOSRd:
  hasOSRd = no.
  find _file no-lock where _file._file-name = "_tablestat" no-error.
  if available _file then find _field no-lock of _file where _field._field-name = "_tablestat-osread" no-error.
  hasOSRd = available( _field ).
  return.
end.


define variable tblOffset as integer no-undo.

procedure mon-init:

  define variable b as handle.

  b = ( temp-table tt_tblAct:default-buffer-handle ).

  if pt_altWidth = "80" then
    do:
      altFormat( "80", b, "tblName",  "x(30)" ).
      altFormat( "80", b, "tblTurn",  ">>>>>9.99" ).
      altFormat( "80", b, "tblCr",    ">>>>>>>>9" ).
      altFormat( "80", b, "tblRd",    ">>>>>>>>9" ).
      altFormat( "80", b, "tblUp",    ">>>>>>>>9" ).
      altFormat( "80", b, "tblDl",    ">>>>>>>>9" ).
    end.


  empty temp-table tt_table.

  chkTblRange().

  /*** Prior to 11.7 _TableStat-Id mapping to _File-num is "quirky" when _statbase._tablebase <> 1, ditto for _Indexstat
   ***
   ***/

  find _tablestat no-lock where _tablestat-id = 1 no-error.

  /***
   * display _tablestat.	/* prior to 11.7 _tablestat-id will magically change to whatever table is mapped to slot #1	*/
   * pause.
   ***/

  if available( _tablestat ) and _tablestat-id <> 1 then
    tblOffset = ( _tablestat-id * -1 ) + 1.	/* this is the offset that needs to be applied to real table numbers to map to _tablestat	*/

  /*** 
   * find _tablestat no-lock where _tablestat-id = ( tblOffset + 1 ).  /* in this case "1" is the target _file-number */
   * display _tablestat.
   * pause.
   */


  run chkOSRd.

  run updTick.

  return.

end.


procedure mon-update:

  define input parameter argList as character no-undo.

  define variable i        as integer no-undo.
  define variable topLimit as integer no-undo initial 50.				/* formerly 10				*/

  if chkTblRange() = no then
    do:
      run mon-init.
    end.

  run updTick.

  for each tt_tbl:
    run upd-tt_table ( tt_tbl.xid + tblOffset ).
  end.

  /* run age_table. */									/* Does nothing but waste effort...	*/

  empty temp-table tt_tblAct.

  /* top X creates
   */

  i = 0.
  for each tt_table no-lock by tt_table.tbl-cre[x] descending.

    i = i + 1.

    find tt_tblAct where tt_tblAct.tblNum = tt_table.tblNum no-error.			/* prevent duplicates			*/
    if available tt_tblAct then next.

    create tt_tblAct.
    assign
      tt_tblAct.xid        = tt_table.tblNum
      tt_tblAct.tblNum     = tt_table.tblNum
      tt_tblAct.tblAreaNum = tt_table.areaNum
      tt_tblAct.tblPool    = tt_table.tblPool
      tt_tblAct.tblName    = tt_table.tblName
      tt_tblAct.tblCr      = ( tt_table.tbl-cre[x]  / z )
      tt_tblAct.tblRd      = ( tt_table.tbl-rd[x]   / z )
      tt_tblAct.tblUp      = ( tt_table.tbl-upd[x]  / z )
      tt_tblAct.tblDl      = ( tt_table.tbl-del[x]  / z )
      tt_tblAct.tblOSRd    = ( tt_table.tbl-osrd[x] / z )
    .

    run getTableInfo(
      tt_table.tblName,
      output tt_tblAct.dbaRecs,
      output tt_tblAct.dbaRM,
      output tt_tblAct.dbaFragPct,
      output tt_tblAct.dbaScatter,
      output tt_tblAct.dbaAvgRow
    ).
    if tt_tblAct.dbaRecs > 0 then tt_tblAct.tblTurn = tt_tblAct.tblRd / tt_tblAct.dbaRecs.

    if ( topLimit > 0 ) and ( i >= topLimit ) then leave.

  end.

  /* top X reads
   */

  i = 0.
  for each tt_table no-lock by tt_table.tbl-rd[x] descending.

    i = i + 1.

    find tt_tblAct where tt_tblAct.tblNum = tt_table.tblNum no-error.			/* prevent duplicates			*/
    if available tt_tblAct then next.

    create tt_tblAct.
    assign
      tt_tblAct.xid        = tt_table.tblNum
      tt_tblAct.tblNum     = tt_table.tblNum
      tt_tblAct.tblAreaNum = tt_table.areaNum
      tt_tblAct.tblPool    = tt_table.tblPool
      tt_tblAct.tblName    = tt_table.tblName
      tt_tblAct.tblCr      = ( tt_table.tbl-cre[x]  / z )
      tt_tblAct.tblRd      = ( tt_table.tbl-rd[x]   / z )
      tt_tblAct.tblUp      = ( tt_table.tbl-upd[x]  / z )
      tt_tblAct.tblDl      = ( tt_table.tbl-del[x]  / z )
      tt_tblAct.tblOSRd    = ( tt_table.tbl-osrd[x] / z )
    .

    run getTableInfo(
      tt_table.tblName,
      output tt_tblAct.dbaRecs,
      output tt_tblAct.dbaRM,
      output tt_tblAct.dbaFragPct,
      output tt_tblAct.dbaScatter,
      output tt_tblAct.dbaAvgRow
    ).
    if tt_tblAct.dbaRecs > 0 then tt_tblAct.tblTurn = tt_tblAct.tblRd / tt_tblAct.dbaRecs.

    if ( topLimit > 0 ) and ( i >= topLimit ) then leave.

  end.

  /* top X updates
   */

  i = 0.
  for each tt_table no-lock by tt_table.tbl-upd[x] descending.

    i = i + 1.

    find tt_tblAct where tt_tblAct.tblNum = tt_table.tblNum no-error.			/* prevent duplicates			*/
    if available tt_tblAct then next.

    create tt_tblAct.
    assign
      tt_tblAct.xid        = tt_table.tblNum
      tt_tblAct.tblNum     = tt_table.tblNum
      tt_tblAct.tblAreaNum = tt_table.areaNum
      tt_tblAct.tblPool    = tt_table.tblPool
      tt_tblAct.tblName    = tt_table.tblName
      tt_tblAct.tblCr      = ( tt_table.tbl-cre[x]  / z )
      tt_tblAct.tblRd      = ( tt_table.tbl-rd[x]   / z )
      tt_tblAct.tblUp      = ( tt_table.tbl-upd[x]  / z )
      tt_tblAct.tblDl      = ( tt_table.tbl-del[x]  / z )
      tt_tblAct.tblOSRd    = ( tt_table.tbl-osrd[x] / z )
    .

    run getTableInfo(
      tt_table.tblName,
      output tt_tblAct.dbaRecs,
      output tt_tblAct.dbaRM,
      output tt_tblAct.dbaFragPct,
      output tt_tblAct.dbaScatter,
      output tt_tblAct.dbaAvgRow
    ).
    if tt_tblAct.dbaRecs > 0 then tt_tblAct.tblTurn = tt_tblAct.tblRd / tt_tblAct.dbaRecs.

    if ( topLimit > 0 ) and ( i >= topLimit ) then leave.

  end.

  /* top X deletes
   */

  i = 0.
  for each tt_table no-lock by tt_table.tbl-del[x] descending.

    i = i + 1.

    find tt_tblAct where tt_tblAct.tblNum = tt_table.tblNum no-error.			/* prevent duplicates			*/
    if available tt_tblAct then next.

    create tt_tblAct.
    assign
      tt_tblAct.xid        = tt_table.tblNum
      tt_tblAct.tblNum     = tt_table.tblNum
      tt_tblAct.tblAreaNum = tt_table.areaNum
      tt_tblAct.tblPool    = tt_table.tblPool
      tt_tblAct.tblName    = tt_table.tblName
      tt_tblAct.tblCr      = ( tt_table.tbl-cre[x]  / z )
      tt_tblAct.tblRd      = ( tt_table.tbl-rd[x]   / z )
      tt_tblAct.tblUp      = ( tt_table.tbl-upd[x]  / z )
      tt_tblAct.tblDl      = ( tt_table.tbl-del[x]  / z )
      tt_tblAct.tblOSRd    = ( tt_table.tbl-osrd[x] / z )
    .

    run getTableInfo(
      tt_table.tblName,
      output tt_tblAct.dbaRecs,
      output tt_tblAct.dbaRM,
      output tt_tblAct.dbaFragPct,
      output tt_tblAct.dbaScatter,
      output tt_tblAct.dbaAvgRow
    ).
    if tt_tblAct.dbaRecs > 0 then tt_tblAct.tblTurn = tt_tblAct.tblRd / tt_tblAct.dbaRecs.

    if ( topLimit > 0 ) and ( i >= topLimit ) then leave.

  end.

  add2ds( temp-table tt_tblAct:default-buffer-handle ).

  return.

end.

{ssg/recact.i}

return.
