/* updorder.p
 *
 * randomly update an order's salesRep
 *
 */

define variable flgName as character no-undo initial "updorder&1.flg".
define variable repList as character no-undo.

define variable i as integer no-undo.
define variable j as integer no-undo.
define variable k as integer no-undo.
define variable n as integer no-undo.
define variable r as integer no-undo.

define variable firstOrder as integer no-undo.
define variable lastOrder  as integer no-undo.
define variable lastTime   as integer no-undo.

flgName = substitute( flgName, session:parameter ).
output to value( flgName ) unbuffered append.
message now flgName.
output close.

for each salesRep no-lock:
  repList = repList + "," +  salesRep.salesRep.
  n = n + 1.
end.

repList = trim( repList, "," ).

find first order no-lock.
firstOrder = order.orderNum.

find last order no-lock.
lastOrder = order.orderNum.

loop: do while true:

  /* the default sports2000 db has a bit fewer than 4,000 orders so we can update
   * roughly one every 20 seconds and not run out, as the day wears on we will be
   * more and more likely to be updating a record that has already been updated
   * so the rate of "new" orders will decrease over time
   */

  r = random( 2, 5 ).		/* minimum of 2 seconds, average 6 seconds */

  /* the FOR EACH ORDER below takes about 90 seconds on my slow and misconfigured test server
   * so it is unlikely that the time governor will come into play.  but if you run on a faster
   * or better configured server it could be necessary
   */ 

  if abs( time - lastTime ) < r then
    assign
      r = ?
      i = ?
    .
   else
    do:
      lastTime = time.
      r = random( 1, 5 ).
      do k = 1 to r.
        newOrder: do i = 1 to 100:
          j = random( firstOrder, lastOrder ).
          find order no-lock where order.orderNum = j no-error.
          if available order and order.orderDate <> today then
            do:
              run updOrderDate( j ).
              leave newOrder.
            end.
        end.
      end.
    end.

  /* generate a bunch of bi notes by updating the salesRep
   * and filling the instructions with garbage text
   */

  for each order no-lock:

    run updSalesRep( order.orderNum, entry( random( 1, n ), repList )).

    file-info:file-name = flgName.
    if file-info:full-pathname = ? then leave loop.

  end.

  message now.

end.

quit.


/* pretend to get new orders
 */

procedure updOrderDate:

  define input parameter oNum as integer   no-undo.

  define buffer order for order.
  define buffer updOrder for order.

  do for updOrder transaction:

    find updOrder exclusive-lock where updOrder.orderNum = oNum no-wait no-error.
    if available updOrder then
      do:
        assign
          updOrder.orderDate = today
        .
      end.

  end.

  return.

end.


/* generate a bunch of bi notes
 */

procedure updSalesRep:

  define input parameter oNum as integer   no-undo.
  define input parameter sRep as character no-undo.

  define buffer order for order.
  define buffer updOrder for order.

  define variable i as integer no-undo.

  do for updOrder transaction:

    /* for our purposes there is no point in waiting -- we just want to generate a bunch of bi notes
     */

    find updOrder exclusive-lock where updOrder.orderNum = oNum no-wait no-error.
    if available updOrder then
      do:
        assign
          updOrder.salesRep = sRep
          updOrder.instructions = ""
        .
        do i = 1 to random( 1, 8192 ):
          updOrder.instructions = updOrder.instructions + chr( random( 32, 122 )).
        end.
      end.

    /* we don't want to go completely crazy -- just "crazy enough to generate noticeable update activity"
     */

    pause 0.01.		/* this also increases the chances that the "curr bi cluster" will reflect this activity	*/

  end.

  return.

end.
