#!/bin/sh

THIS=`basename $0`
Release="V2.1 as of Apr 20, 2019"

# masterblock.sh - shell script to read the values of the fields in
#                  master block of Progress database.
#
# The changes in V2.1 as of Apr 20, 2019:
#  dd if=$2 bs=1 skip=$Offset count=<N> 2>/dev/null | od -<X>
# are replaced by
#  od -<X> -j $Offset -N <N> $2
# because the output od command can be intermittently truncated when the bs flag
# is not "be a multiple of the physical block size for the media being used".
 
mbFields="dbkey type frchn incr  nextf updctr
dbvers dbRestrictVer dbMinorVers clientMinorVers dbstate cfilnum
tainted flags largeFilesEnabled XAsupportEnabled areaHWM lasttask
replai.aibegin replai.ainew replai.aigennbr replai.aiopen repllstmod replupdctr
ai.aibegin ai.ainew ai.aigennbr ai.aiopen lstmod aiwrtloc aictr aiflgs wrd
wdcrc biopen biprev rltime rlclsize bhvolno aisync chgd bistate langok
br_language br_usrfl aiseq aiSequence buovl buseq crdate oprdate oppdate fbdate
ibdate ibseq dbcnt bicnt stamp democt flag nicknm lightai crd replEnabled
pdrEnabled replRET tlwrtloc tlwrtoffset tlwrtloc64 aiwrtloc64 biblksize
aiblksize maxArgnclts txnSequence lastLogOpSeq fmaDBContent fmaDBEnabled
spathnm aicnt aibusy_extent seqblk dbrecid shutdown timestamp shmid semid
objectRoot replFlags dsrCRAISeq dsrCRLoc dsrCROffset dsrRCBlock objSerialNum
fmaEnabledCRC fmaContentCRC maxcluster clustEnabled SaveKeyEvents
noname1 noname2 noname3"

mbFields=`echo $mbFields`

# ProTop customization
. #ENVPT3#/bin/protopenv

#------------------------------------------------------------------------------

Usage()
{
  echo "$BD$THIS$UB: read the fields from master block, Release $Release

Usage: 
$THIS dbname [field1 ...]
$THIS -h|-help

where:
  ${UL}dbname$UU specifies a database whose master block you want to read;
  ${UL}fieldN$UU specifies a field in master block whose values you want to read.
  Field names can be specified with or without \"bk_\" or \"mb_\" prefixes;
  -h is help. Print the names of all fields known by the script.
" # echo
} # Usage()


#------------------------------------------------------------------------------

FullOffset()
{
# $1 is a field offset from the beginning of block.
# Field offset from the beginning of file is:
#
  Offset=`expr $BlockOffset + $1`

# Correction of field offset on Linux:
#
  case `uname` in
   Linux)
     if [ $1 -gt 660 ]
     then
       Offset=`expr $Offset - 8`
     else
       test $1 -gt 220 && \
       Offset=`expr $Offset - 4`
     fi
    ;; # Linux
  esac # case uname

  echo $Offset

} # FullOffset()

#------------------------------------------------------------------------------

GetDec1()
{
  Offset=`FullOffset $1`
# $2 is an input file

# od -d outputs the unsigned decimal 2-byte units

  od -d -j $Offset -N 1 $2 | \
  awk 'NF<2 {print "ERROR"
             exit
       } # NF<2

# echo "" | od -d -j 0 -N 1
# Solaris: 0000000 02560 (=10*256)
# Linux:   0000000    10 (="\n")
# GetDec1 should return 10

       {if($2>=256) print $2 / 256
        else        print $2 + 0
        exit
       } # NF>=2
  ' # awk

} # GetDec1()

#------------------------------------------------------------------------------

GetHex1()
{
  Offset=`FullOffset $1`
# $2 is an input file

# od -x outputs the hexadecimal 2-byte units

  od -x -j $Offset -N 1 $2 | \
  awk 'NF<2 {print "ERROR"
             exit
       } # NF<2

# echo "12345678" | dd bs=1 count=1 | od -x
# Solaris: 0000000 3100
# Linux:   0000000 0031
# GetHex1 should return 0x31

       {if(substr($2,1,2)=="00") print "0x" substr($2,3,2)
        else                     print "0x" substr($2,1,2)
        exit
       } # NF>=2
  ' # awk

} # GetHex1()

#------------------------------------------------------------------------------

GetDec2()
{
  Offset=`FullOffset $1`
# $2 is an input file

# od -d outputs the unsigned decimal 2-byte units

  od -d -j $Offset -N 2 $2 | \
  awk 'NF<2 {print "ERROR"
             exit
       } # NF<2

       {print $2+0
        exit
       } # NF>=2
  ' # awk

} # GetDec2()

#------------------------------------------------------------------------------

GetHex2()
{
  Offset=`FullOffset $1`
# $2 is an input file

# od -x outputs the hexadecimal 2-byte units

  od -x -j $Offset -N 2 $2 | \
  awk 'NF<2 {print "ERROR"
             exit
       } # NF<2

       {print "0x" $2
        exit
       } # NF>=2
  ' # awk

} # GetHex2()

#------------------------------------------------------------------------------

GetDec4()
{
  Offset=`FullOffset $1`
# $2 is an input file

# od -d outputs the unsigned decimal 2-byte units
# GetDec4 returns the unsigned long words

  od -d -j $Offset -N 4 $2 | \
  awk 'NF<3 {print "ERROR"
             exit
       } # NF<3

       {if("'$Endian'"=="Big") print $3 + $2 * 65536
        else                   print $2 + $3 * 65536
         exit
       }
   ' # awk

} # GetDec4()

#------------------------------------------------------------------------------

GetHex4()
{
  Offset=`FullOffset $1`
# $2 is an input file

# od -x outputs the hexadecimal 2-byte units

  od -x -j $Offset -N 4 $2 | \
  awk 'NF<3 {print "ERROR"
             exit
      } # NF<3

      {if("'$Endian'"=="Big") print "0x" $2 $3
       else                   print "0x" $3 $2
       exit
      }
  ' # awk

} # GetHex4()

#------------------------------------------------------------------------------

GetDec8()
{
  Offset=`FullOffset $1`
# $2 is an input file

# od -d outputs the unsigned decimal 2-byte units

  od -d -j $Offset -N 8 $2 | \
  awk 'NF<5 {print "ERROR"
             exit
      } # NF<5

      {Dec=0
       if("'$Endian'"=="Big") for(i=2;i<=5;i++) Dec=Dec*65536+$i
       else                   for(i=5;i>=2;i--) Dec=Dec*65536+$i
       print Dec
       exit
      } # NF>=5
  ' # awk

} # GetDec8()

#------------------------------------------------------------------------------

GetHex8()
{
  Offset=`FullOffset $1`
# $2 is an input file

# od -x outputs the hexadecimal 2-byte units

  od -x -j $Offset -N 8 $2 | \
  awk 'NF<5 {print "ERROR"
             exit
      } # NF<5

      {Hex=""
       if("'$Endian'"=="Big") {for(i=2;i<=5;i++) Hex=Hex $i}
       else                   {for(i=5;i>=2;i--) Hex=Hex $i}
       print "0x" Hex
       exit
      } # NF>=5
  ' # awk

} # GetHex8()

#------------------------------------------------------------------------------

GetTime()
{
  Time=`GetDec4 $1 $2`

  if [ $Time -eq 0 ]
  then
    echo "$Time (--none--)"
    return
  fi # if $Time -eq 0

# Linux, AIX but not SunOS, HP-UX:
# The --date=STRING is a mostly free format human readable date string
  Date=`date --date @$Time 2>/dev/null`
  if [ "$Date" ]
  then
    echo "$Time ($Date)"
  else
    echo "$Time (seconds since Jan 01, 1970)"
  fi # if "$Date"

} # GetTime()

#------------------------------------------------------------------------------

GetChar()
{
  Offset=`FullOffset $1`
# $2 is an input file

# Return a string terminated by any non-printable character (according to od).
# Note: master block stores the null terminated strings.
#
  i1=`echo AB | od -c | awk '{print index($0,"A"); exit}'`
  i2=`echo AB | od -c | awk '{print index($0,"B"); exit}'`
  i2=`expr $i2 - $i1`

  od -c -j $Offset -N 256 $2 | \
  awk '{n=length($0)
        for(i='$i1'; i<=n; i+='$i2')
         if(substr($0,i-1,1)==" ") printf"%s",substr($0,i,1)
         else exit
       }' -
  # awk

} # GetChar()

#------------------------------------------------------------------------------
#

# TERM is not set if the script is running by cron:
if [ "$TERM" ]
then
# Bold/unbold terminal codes:
  BD=`tput smso` # Enter standout mode (bold on rxvt)
  UB=`tput rmso` # Exit standout mode

# Underline mode:
  UL=`tput smul` # Begin underline mode
  UU=`tput rmul` # Exit underline mode
fi

ValidField=`echo $ValidField`

# Check input parameters: -----------------------------------------------------

# No input parameters:
#
if [ $# -eq 0 ]
then
  Usage
  exit
fi # if $# -eq 0

# The -help option:
#
case $1 in
  -h*|--help) Usage
    echo "The list of fields in master block:"
    echo $mbFields | \
    awk 'BEGIN {RS=FS}
         {printf"%17s ",$1}
         NR % 4 == 0 {printf"\n"}
         END {printf"\n"}
    ' # awk
      exit
  ;; # -help
esac # case $1

# Db name:
#
DB=`echo $1 | sed -e 's/\.db$//'`

if [ ! -f $DB.db ]
then
  echo "$THIS: Database $BD$DB$UB not found!" >&2
  exit 1
fi # if ! -f $DB.db

if [ ! -r $DB.db ]
then
  echo "$THIS: You do not have permissions to read $BD$DB.db$UB" >&2
  exit 1
fi # if ! -f $DB.db

shift

# Validate the input list of the field names:
#
InputFields=""
WrongFields=""
while [ $# -gt 0 ]
do
  ShortName=`echo $1 | sed -e 's/^mb_//'`
  echo " $mbFields " | grep " $ShortName " >/dev/null && \
  InputFields="$InputFields $1" || \
  WrongFields="$WrongFields $1"
  shift
done  # while $# -gt 0

if [ "$WrongFields" ]
then
  echo "$THIS: Unknown field(s): $WrongFields
Use -help to get the list of the fields in master block." >&2
  exit 1
fi # if $WrongFields

InputFields=${InputFields:-$mbFields}

Value=`echo 01 | od -x | awk 'NF>1 {print $2}'`
case $Value in
# The most significant byte (MSB) first:
 3031*) Endian="Big";;
# The least significant byte (LSB) in the lowest memory address:
 3130*) Endian="Little";;
     *) echo "$THIS: failed to check OS endian (ret: $Value)." >&2
        exit 1;;
esac

# Extent Header Block (EXTHDR, bk_type 254)
#
#   Offset Size Field Name    Description
# --------- --- ------------- ------------------------------------------------
# 16 0x0010   2 dbVersion     Database version number (high bite is blocksize)
# 18 0x0012   2 extentType
# 22 0x0016   2 dateVerification
# 24 0x0018 2*4 creationDate
# 32 0x0020 2*4 lastOpenDate

# Extent Header Block is the first block in $DB.db:
#
BlockOffset=0
dbVersion=`GetDec2 16 $DB.db`
DbVersion=`expr $dbVersion % 256`
DbBlockSize=`expr $dbVersion - $DbVersion`

# echo debug: DbVersion=$DbVersion >&2
# echo debug: DbBlockSize=$DbBlockSize >&2
# echo debug: OS \(`uname`\) is $Endian Endian. >&2

case $DbBlockSize in
  1024) ;;
  2048) ;;
  4096) ;;
  8192) ;;
     *)
    echo "$THIS: Wrong db blocksize: $DbBlockSize
dbVersion in Extent Header of $DB.db is $dbVersion
OS (`uname`) is $Endian Endian." >&2
    exit 1
  ;; # Wrong db blocksize
esac # case $DbBlockSize


# Control Block (CTLBLK, bk_type: 13)
#
#   Offset Size Field Name     Description
# --------- --- -------------- -----------------------------------------------
# 16 0x0010   8 areaRoot       Dbkey of root block of _Area._Area-Number
# 24 0x0018   8 areaExtentRoot Dbkey of root block of _AreaExtent._AreaExtent-Area
# 32 0x0020 256 d1PathName     Path to a database extent containing master block

BlockOffset=$DbBlockSize
d1PathName=`GetChar 32 $DB.db`

if [ -z "$d1PathName" ]
then
  echo "$THIS: Failed to get d1PathName." >&2
  exit 1
fi # if -z $d1PathName

# echo debug: d1PathName=$d1PathName >&2

d1Path=$d1PathName
case $d1PathName in
 ./*) d1Path=`dirname $DB`/`basename $d1PathName`;;
esac

# echo debug: d1Path=$d1Path >&2

if [ ! -f $d1Path ]
then
  echo "$THIS: Cannot find $BD$d1Path$UB.
d1PathName in Control Block of $DB.db is $d1PathName" >&2
  exit 1
fi # if ! -f $d1Path

if [ ! -r $d1Path ]
then
  echo "$THIS: You do not have permissions to read $BD$d1Path$UB" >&2
  exit 1
fi # if ! -f $d1Path

# Master Block is the second block in $d1Path:
#
BlockOffset=$DbBlockSize

for Field in $InputFields
do
  ShortName=`echo $Field | sed -e 's/^mb_//'`
  ShortName=`echo $Field | sed -e 's/^bk_//'`

  case $ShortName in
    br_language) FullName=$ShortName;;
       br_usrfl) FullName=$ShortName;;
          dbkey) FullName=bk_$ShortName;;
           type) FullName=bk_$ShortName;;
          frchn) FullName=bk_$ShortName;;
           incr) FullName=bk_$ShortName;;
          nextf) FullName=bk_$ShortName;;
         updctr) FullName=bk_$ShortName;;
              *) FullName=mb_$ShortName;;
  esac # case $ShortName

  case $ShortName in

                dbkey) Value=`GetDec4 000 $d1Path`;; # 0x0000
                 type) Value=`GetDec1 004 $d1Path`;; # 0x0004
                frchn) Value=`GetDec1 005 $d1Path`;; # 0x0005
                 incr) Value=`GetDec2 006 $d1Path`;; # 0x0006
                nextf) Value=`GetDec4 008 $d1Path`;; # 0x0008
               updctr) Value=`GetDec4 012 $d1Path`;; # 0x000C
               dbvers) Value=`GetDec2 016 $d1Path`;; # 0x0010 
        dbRestrictVer) Value=`GetDec2 018 $d1Path`;; # 0x0012 
          dbMinorVers) Value=`GetDec4 020 $d1Path`;; # 0x0014 
      clientMinorVers) Value=`GetDec4 024 $d1Path`;; # 0x0018 
              dbstate) Value=`GetDec2 028 $d1Path`;; # 0x001C 
              cfilnum) Value=`GetDec2 030 $d1Path`;; # 0x001E 
              tainted) Value=`GetDec2 032 $d1Path`;; # 0x0020 
                flags) Value=`GetDec2 034 $d1Path`;; # 0x0022 
        SaveKeyEvents) Value=`GetDec1 037 $d1Path`;; # 0x0025 
    largeFilesEnabled) Value=`GetDec1 038 $d1Path`;; # 0x0026 
     XAsupportEnabled) Value=`GetDec1 039 $d1Path`;; # 0x0027 
              areaHWM) Value=`GetDec4 040 $d1Path`;; # 0x0028 
             lasttask) Value=`GetDec4 044 $d1Path`;; # 0x002C 
       replai.aibegin) Value=`GetTime 048 $d1Path`;; # 0x0030 
         replai.ainew) Value=`GetTime 052 $d1Path`;; # 0x0034 
      replai.aigennbr) Value=`GetDec4 056 $d1Path`;; # 0x0038 
        replai.aiopen) Value=`GetTime 060 $d1Path`;; # 0x003C 
           repllstmod) Value=`GetTime 064 $d1Path`;; # 0x0040 
           replupdctr) Value=`GetDec4 068 $d1Path`;; # 0x0044 
           tlwrtloc64) Value=`GetDec8 072 $d1Path`;; # 0x0048 
           aiSequence) Value=`GetDec4 080 $d1Path`;; # 0x0050 
           ai.aibegin) Value=`GetTime 088 $d1Path`;; # 0x0058 
             ai.ainew) Value=`GetTime 092 $d1Path`;; # 0x005C 
          ai.aigennbr) Value=`GetDec4 096 $d1Path`;; # 0x0060 
            ai.aiopen) Value=`GetTime 100 $d1Path`;; # 0x0064 
               lstmod) Value=`GetTime 104 $d1Path`;; # 0x0068 
             aiwrtloc) Value=`GetDec4 108 $d1Path`;; # 0x006C 
                aictr) Value=`GetDec4 112 $d1Path`;; # 0x0070 
               aiflgs) Value=`GetHex1 116 $d1Path`;; # 0x0074 
                  wrd) Value=`GetDec1 117 $d1Path`;; # 0x0075 
                wdcrc) Value=`GetDec2 118 $d1Path`;; # 0x0076 
               biopen) Value=`GetTime 120 $d1Path`;; # 0x0078 
               biprev) Value=`GetTime 124 $d1Path`;; # 0x007C 
               rltime) Value=`GetDec4 128 $d1Path`;; # 0x0080 
             rlclsize) Value=`GetDec2 132 $d1Path`;; # 0x0084 
              bhvolno) Value=`GetDec2 134 $d1Path`;; # 0x0086 
               aisync) Value=`GetDec1 136 $d1Path`;; # 0x0088 
                 chgd) Value=`GetDec1 137 $d1Path`;; # 0x0089 
              bistate) Value=`GetHex1 138 $d1Path`;; # 0x008A 
               langok) Value=`GetDec1 139 $d1Path`;; # 0x008B 
          br_language) Value=`GetDec1 140 $d1Path`;; # 0x008C 
             br_usrfl) Value=`GetDec1 141 $d1Path`;; # 0x008D 
                aiseq) Value=`GetDec2 142 $d1Path`;; # 0x008E 
                buovl) Value=`GetDec2 144 $d1Path`;; # 0x0090 
                buseq) Value=`GetDec2 146 $d1Path`;; # 0x0092 
               crdate) Value=`GetTime 148 $d1Path`;; # 0x0094 
              oprdate) Value=`GetTime 152 $d1Path`;; # 0x0098 
              oppdate) Value=`GetTime 156 $d1Path`;; # 0x009C 
               fbdate) Value=`GetTime 160 $d1Path`;; # 0x00A0 
               ibdate) Value=`GetTime 164 $d1Path`;; # 0x00A4 
                ibseq) Value=`GetDec4 168 $d1Path`;; # 0x00A8 
                dbcnt) Value=`GetDec2 172 $d1Path`;; # 0x00AC 
                bicnt) Value=`GetDec2 174 $d1Path`;; # 0x00AE 
                stamp) Value=`GetHex8 176 $d1Path`;; # 0x00B0 
               democt) Value=`GetDec2 184 $d1Path`;; # 0x00B8 
                 flag) Value=`GetDec2 186 $d1Path`;; # 0x00BA 
               nicknm) Value=`GetChar 188 $d1Path`;; # 0x00BC 
              lightai) Value=`GetDec1 200 $d1Path`;; # 0x00C8 
                  crd) Value=`GetDec1 201 $d1Path`;; # 0x00C9 
          replEnabled) Value=`GetDec1 202 $d1Path`;; # 0x00CA 
           pdrEnabled) Value=`GetDec1 203 $d1Path`;; # 0x00CB 
              replRET) Value=`GetDec4 204 $d1Path`;; # 0x00CC 
             tlwrtloc) Value=`GetDec4 212 $d1Path`;; # 0x00D4 
          tlwrtoffset) Value=`GetDec4 216 $d1Path`;; # 0x00D8 
# On Linux shift left by 4 bytes: see FullOffset()
           aiwrtloc64) Value=`GetDec8 224 $d1Path`;; # 0x00E0 
            biblksize) Value=`GetDec2 232 $d1Path`;; # 0x00E8 
            aiblksize) Value=`GetDec2 234 $d1Path`;; # 0x00EA 
          maxArgnclts) Value=`GetDec4 236 $d1Path`;; # 0x00EC 
          txnSequence) Value=`GetDec8 240 $d1Path`;; # 0x00F0 
         lastLogOpSeq) Value=`GetDec8 248 $d1Path`;; # 0x00F8 
         fmaDBContent) Value=`GetHex8 256 $d1Path`;; # 0x0100 
         fmaDBEnabled) Value=`GetHex8 264 $d1Path`;; # 0x0108 
              noname1) Value=`GetDec1 272 $d1Path`;; # 0x0110 
              noname2) Value=`GetDec4 276 $d1Path`;; # 0x0114 
              noname3) Value=`GetDec8 280 $d1Path`;; # 0x0118 
              spathnm) Value=`GetChar 364 $d1Path`;; # 0x0168 
                aicnt) Value=`GetDec2 620 $d1Path`;; # 0x026C 
        aibusy_extent) Value=`GetDec2 622 $d1Path`;; # 0x026E 
               seqblk) Value=`GetDec4 624 $d1Path`;; # 0x0270 
              dbrecid) Value=`GetDec4 628 $d1Path`;; # 0x0274 
             shutdown) Value=`GetDec4 632 $d1Path`;; # 0x0278 
            timestamp) Value=`GetTime 636 $d1Path`;; # 0x027C 
                shmid) Value=`GetDec4 640 $d1Path`;; # 0x0280 
                semid) Value=`GetDec4 644 $d1Path`;; # 0x0284 
           objectRoot) Value=`GetDec4 648 $d1Path`;; # 0x0288 
            replFlags) Value=`GetHex4 652 $d1Path`;; # 0x028C 
           dsrCRAISeq) Value=`GetDec4 656 $d1Path`;; # 0x0290 
# On Linux shift left by 8 bytes: see FullOffset()
             dsrCRLoc) Value=`GetDec8 664 $d1Path`;; # 0x0298 
          dsrCROffset) Value=`GetDec4 672 $d1Path`;; # 0x02A0 
           dsrRCBlock) Value=`GetDec4 676 $d1Path`;; # 0x02A4 
         objSerialNum) Value=`GetDec8 680 $d1Path`;; # 0x02A8 
        fmaEnabledCRC) Value=`GetDec2 688 $d1Path`;; # 0x02B0 
        fmaContentCRC) Value=`GetDec2 690 $d1Path`;; # 0x02B2 
           maxcluster) Value=`GetDec2 692 $d1Path`;; # 0x02B4 
         clustEnabled) Value=`GetDec2 694 $d1Path`;; # 0x02B6 
  esac # case $ShortName

# Add the description to the list values of some fields:
# 
  case $ShortName in

    nicknm|spathnm) Value=\"$Value\";;

    type) # Block Types
      case $Value in
         1) Value="$Value (Master Block)";;
         2) Value="$Value (Index Block)";;
         3) Value="$Value (Data Block)";;
         4) Value="$Value (Free Block)";;
         5) Value="$Value (Index Dbkey List Block)";;
         6) Value="$Value (Sequence Block)";;
         7) Value="$Value (Empty Block)";;
         8) Value="$Value (Parameter Block)";;
         9) Value="$Value (Area Block)";;
        10) Value="$Value (Object Directory Block)";;
        11) Value="$Value (Extent List Block)";;
        12) Value="$Value (Object Block)";;
        13) Value="$Value (Control Block)";;
        14) Value="$Value (Area Object List Block)";;
        15) Value="$Value (Area Cluster Allocation Block)";;
        16) Value="$Value (Object Cluster List Block)";;
        17) Value="$Value (Object Block Allocation Block)";;
        18) Value="$Value (RM bit Map List Block)";;
        19) Value="$Value (Extent Header Block)";;
        20) Value="$Value (Reserved Block)";;
      esac # case $Value
    ;; # type


    frchn) # Chain Types
      case $Value in
         0) Value="$Value (FREECHN)";;
         1) Value="$Value (RMCHN)";;
         2) Value="$Value (LOCKCHN)";;
         3) Value="$Value (CLIST_FREECHN)";;
       127) Value="$Value (NOCHN)";;
      esac # case $Value
    ;; # frchn


    dbvers)
      dbVersion=`expr $Value % 256`
      dbBlockSize=`expr $Value - $dbVersion`

      case $dbVersion in
        80) dbVersion="$dbVersion=V8";;
        83) dbVersion="$dbVersion=V8.3";;
        91) dbVersion="$dbVersion=V9";;
       150) dbVersion="$dbVersion=V10";;
       173) dbVersion="$dbVersion=V11";;
       183) dbVersion="$dbVersion=V12";;
      esac # case $dbVersion

      Value="$Value (version: $dbVersion, dbblocksize: $dbBlockSize)"
    ;; # dbvers


    dbstate)
      Dec=$Value
      Num=0
      while [ $Dec -gt 0 ]
      do
        Num=`expr $Num + 1`
        Bit=`expr $Dec % 2`
        Dec=`expr $Dec - $Bit`
        Dec=`expr $Dec / 2`
        test $Bit -eq 1 && \
        case $Num in
           1) Value="$Value DBOPEN";;
           2) Value="$Value DBCLOSED";;
           3) Value="$Value DBRECOV";;
           4) Value="$Value DBRESTORE";;
           5) Value="$Value DBVOID";;
           6) Value="$Value DBMSTR";;
           7) Value="$Value DBMDB";;
           8) Value="$Value DBMBI";;
           9) Value="$Value DBMAI";;
          10) Value="$Value DBMTL";;
          11) Value="$Value DBCONVCH";;
          12) Value="$Value DBIXRPR";;
          13) Value="$Value bit13";;
          14) Value="$Value bit14";;
          15) Value="$Value bit15";;
          16) Value="$Value bit16";;
        esac # case $Num in
      done # while $Dec -gt 0
    ;; # dbstate

    tainted|flags)
      Dec=$Value
      Num=0
      while [ $Dec -gt 0 ]
      do
        Num=`expr $Num + 1`
        Bit=`expr $Dec % 2`
        Dec=`expr $Dec - $Bit`
        Dec=`expr $Dec / 2`
        test $Bit -eq 1 && \
        case $Num in
           1) Value="$Value ARGFORCE";;
           2) Value="$Value NOINTEG";;
           3) Value="$Value NONRAW";;
           4) Value="$Value DBRESTORE";;
           5) Value="$Value DBBACKUP";;
           6) Value="$Value DBIXBUILD";;
           7) Value="$Value DBROLL_NOINTEG";;
           8) Value="$Value DBROLL_PARTIAL";;
           9) Value="$Value DBADD_ONLINE";;
          10) Value="$Value DBVSTS_UPDATED";;
          11) Value="$Value DBSEQ64_ENABLED";;
          12) Value="$Value ENABLEAI_ONLINE";;
          13) Value="$Value DBROLL_STARTED";;
          14) Value="$Value DBREDO_CLEAN";;
          15) Value="$Value DBSKIP_RECOVER";;
          16) Value="$Value DBKS_REBIND";;        
        esac # case $Num in
      done # while $Dec -gt 0
    ;; # tainted|flags

    bistate)
      case $Value in
        0x13) Value="$Value BITRUNC";;
        0x14) Value="$Value BIEXIST";;
      esac # case $Value in
    ;; # bistate

    aiflgs)
      case $Value in
        0x01) Value="$Value MBAIAUDIT";;
      esac # case $Value in
    ;; # aiflgs

    flag)
      Dec=$Value
      Num=0
      while [ $Dec -gt 0 ]
      do
        Num=`expr $Num + 1`
        Bit=`expr $Dec % 2`
        Dec=`expr $Dec - $Bit`
        Dec=`expr $Dec / 2`
        test $Bit -eq 1 && \
        case $Num in
          1) Value="$Value DEMOVERS";;
          2) Value="$Value UNDECIDED";;
        esac # case $Num in
      done # while $Dec -gt 0
    ;; # flag

   shmid|semid)
     test $Value -eq 4294967295 && \
     Value="$Value (0xFFFFFFFF = not in use)"
   ;; # shmid|semid

# noname2 is an offset in block of Before Image Log Initialisation;
# noname3 is a block of Before Image Log Initialisation:
# BROKER  0: Before Image Log Initialisation at block 19  offset 5663. (15321)
# BROKER  0: (15743) Before Image Log Completion at Block 19 Offset 5663.

    noname2)
      Value="$Value (offset in block of Before Image Log Initialisation)"
    ;; # noname2

    noname3)
      Value="$Value (block of Before Image Log Initialisation)"
    ;; # noname3

  esac # case $ShortName
   
  echo $FullName $Value | \
  awk '{printf"%-21s %s\n",$1,substr($0,length($1)+2)}'

done # for Field in $InputFields

