/* util/maintenance.p 
 *
 * mark b 040720
 *
 */

/* define variables to accept the params from maintenance.sh */

define variable mode      as character no-undo.
define variable inDur     as character no-undo.
define variable tmpDir    as character no-undo.
define variable logDir    as character no-undo.
define variable log       as character no-undo.
define variable custId    as character no-undo. 
define variable ptHost    as character no-undo.
define variable cmdLine   as character format "x(70)" no-undo.

define variable maintEnd  as DATETIME-TZ format "99-99-9999 HH:MM:SS+HH:MM" no-undo.
define variable maintFile as character no-undo format "x(60)".
define variable qStr      as character no-undo format "x(120)".

assign
  mode      = entry( 1, session:parameter, "|" )
  inDur     = entry( 2, session:parameter, "|" )
  tmpDir    = entry( 3, session:parameter, "|" )
  logDir    = entry( 4, session:parameter, "|" )
  custId    = entry( 5, session:parameter, "|" )
  ptHost    = entry( 6, session:parameter, "|" )
  maintFile = tmpDir + "/MAINTENANCE"
  log       = logDir + "/maintenance.log"
.

run util/restlib.p persistent.

case mode:

   when "on" then do:

      file-info:file-name = maintFile.
      if file-info:full-pathname ne ? then do:

         input from value ( maintFile ) no-echo.
         import unformatted maintEnd.
         input close.

         message "maintenance mode is already ON until" maintEnd view-as alert-box.

         return.

      end.

      /* create the MAINTENANCE file */

      maintEnd = add-interval ( NOW, integer(inDur), "hours" ).
      output to value(maintFile) no-echo.
      put maintEnd skip.
      output close.
 
      /* remove flg file(s) */

      cmdLine = "rm -f " + tmpDir + "/*.flg".
      os-command value( cmdLine ). 

      /* add entry to the log file */
      output to value( log ) append no-echo.
      put now " " maintEnd " ON ".
      put unformatted inDur " hour(s)" skip.
      output close.

      /* post the outage to the dashboard */
      qStr = "siteName=" + custId + "&dbIdList=all&description=Created+by+maintenance.sh&duration=" + inDur + "&oper=add&&req=maintenance".

      output to value( log ) append no-echo.
      run post2portal(                                                          
         qStr,                                                         /* inData      */ 
         ptHost,                                                       /* host        */ 
         80,                                                           /* port        */ 
         '/api/outage.php',                                            /* path        */ 
         '',                                                           /* options     */ 
         '',                                                           /* proxy       */ 
         'Content-Type: application/x-www-form-urlencoded'             /* xtraHeaders */ 
      ).
      output close.

      message "maintenance mode is ON for" inDur "hour(s)" view-as alert-box.

   end. /* mode = on */
   when "off" then do:

      /* if $1 is off then remove the maintenance file */

      file-info:file-name = maintFile.
      if file-info:full-pathname eq ? then

         message "maintenance mode is already OFF" view-as alert-box.

      else do:

         message "ending maintenance mode ..." view-as alert-box.

         /* remove the MAINTENANCE file */

         cmdline = "rm -f " + maintFile.
         os-command value ( cmdLine ).

         file-info:file-name = maintFile.
         if file-info:full-pathname ne ? then

            message "FAILED! maintenance mode is still ON" view-as alert-box.

         else do:

            /* add entries to log file */

            maintEnd = now.

            output to value( log ) append no-echo.
            put now " " maintEnd " OFF" skip.
            output close.

            /* post end of outage to dashboard */ 

            qStr = "siteName=" + custId + "&oper=del&req=maintenance".

            output to value( log ) append no-echo.
            run post2portal( 
               qStr,                                                   /* inData      */
               ptHost,                                                 /* host        */
               '80',                                                   /* port        */
               '/api/outage.php',                                      /* path        */
               '',                                                     /* options     */
               '',                                                     /* proxy       */
               'Content-Type: application/x-www-form-urlencoded'       /* xtraHeaders */
            ).
            output close.

            message "maintenance mode is OFF, protop agents will restart shortly ..." view-as alert-box.
    
         end.

      end.

   end. /* mode = off */

   when "status" then do:

      file-info:file-name = maintFile.
      if file-info:full-pathname eq ? then

         message "maintenance mode is OFF" view-as alert-box.

      else do:

         input from value ( maintFile ) no-echo.
         import unformatted maintEnd.
         input close.

         message "maintenance mode is in effect until" maintEnd view-as alert-box.
 
      end.

   end. /* mode = status */

end case.



/*** finis ***/
